# -*- coding: utf-8 -*-

from doctest import ELLIPSIS
from doctest import NORMALIZE_WHITESPACE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing.layers import FunctionalTesting
from plone.app.testing.layers import IntegrationTesting
from plone.app.z3cform.widget import SelectWidget
from plone.autoform import directives
from plone.autoform.form import AutoExtensibleForm
from plone.testing import z2
from z3c.form import form
from zope.configuration import xmlconfig
from zope.interface import Interface
from zope.interface import directlyProvides
from zope.interface import implementer
from zope.publisher.browser import TestRequest as BaseTestRequest
from zope.schema import Choice
from zope.schema import List
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


@implementer(IVocabularyFactory)
class ExampleVocabulary(object):

    def __call__(self, context, query=None):
        items = [u'One', u'Two', u'Three']
        tmp = SimpleVocabulary([
            SimpleTerm(it.lower(), it.lower(), it)
            for it in items
            if query is None
            or query.lower() in it.lower()
        ])
        tmp.test = 1
        return tmp


def ExampleFunctionVocabulary(context, query=None):
    items = [u'First', u'Second', u'Third']
    tmp = SimpleVocabulary([
        SimpleTerm(it.lower(), it.lower(), it)
        for it in items
        if query is None
        or query.lower() in it.lower()
    ])
    return tmp
directlyProvides(ExampleFunctionVocabulary, IVocabularyFactory)


class TestRequest(BaseTestRequest):
    pass


class DummyContext(object):

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


class DummyATField(object):

    def getName(self):
        return 'dummyname'

    def getAccessor(self, context):
        def accessor():
            return 'dummyvalue'
        return accessor


class PloneAppWidgetsLayer(PloneSandboxLayer):

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plone.app.dexterity
        self.loadZCML(name='meta.zcml', package=plone.app.dexterity)
        self.loadZCML(package=plone.app.dexterity)
        import plone.app.widgets
        self.loadZCML(package=plone.app.widgets)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, 'plone.app.dexterity:default')


PLONEAPPWIDGETS_FIXTURE = PloneAppWidgetsLayer()


class PloneAppWidgetsDXLayer(PloneAppWidgetsLayer):

    def setUpZope(self, app, configurationContext):
        super(PloneAppWidgetsDXLayer, self).setUpZope(app,
                                                      configurationContext)
        import plone.app.contenttypes
        xmlconfig.file('configure.zcml',
                       plone.app.contenttypes,
                       context=configurationContext)

        import plone.app.widgets.tests
        xmlconfig.file('configure.zcml', plone.app.widgets.tests,
                       context=configurationContext)

        try:
            import mockup
            self.loadZCML(package=mockup)
        except:
            pass

        z2.installProduct(app, 'Products.DateRecurringIndex')

    def setUpPloneSite(self, portal):
        super(PloneAppWidgetsDXLayer, self).setUpPloneSite(portal)
        portal.portal_workflow.setDefaultChain("simple_publication_workflow")
        # we need contenttypes before installing widgets
        self.applyProfile(portal, 'plone.app.contenttypes:default')


PLONEAPPWIDGETS_FIXTURE_DX = PloneAppWidgetsDXLayer()

PLONEAPPWIDGETS_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONEAPPWIDGETS_FIXTURE_DX,),
    name="PloneAppWidgetsLayer:Integration")
PLONEAPPWIDGETS_DX_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONEAPPWIDGETS_FIXTURE,),
    name="PloneAppWidgetsLayer:DXIntegration")
PLONEAPPWIDGETS_DX_ROBOT_TESTING = FunctionalTesting(
    bases=(PLONEAPPWIDGETS_FIXTURE_DX,
           REMOTE_LIBRARY_BUNDLE_FIXTURE,
           z2.ZSERVER_FIXTURE),
    name="PloneAppWidgetsLayerDX:Robot")

optionflags = (ELLIPSIS | NORMALIZE_WHITESPACE)


class ITestSelectWidgetSchema(Interface):

    directives.widget('select_field', SelectWidget)
    select_field = Choice(
        title=u'Select Widget',
        values=['one', 'two', 'three', ]
    )

    directives.widget('list_field', SelectWidget)
    list_field = List(
        title=u'Select Multiple Widget',
        value_type=Choice(values=['four', 'five', 'six', ]),
    )


class TestSelectWidgetForm(AutoExtensibleForm, form.EditForm):

    schema = ITestSelectWidgetSchema
    ignoreContext = True



# -*- coding: utf-8 -*-
import zope.deferredimport


zope.deferredimport.deprecated(
    "Import from plone.app.z3cform.widgets.base instead (this package will be removed in Plone 7)",
    NotImplemented='plone.app.z3cform.widgets.base:PatternNotImplemented',
)
zope.deferredimport.deprecated(
    "Import from plone.app.z3cform.widgets.datetime instead (this package will be removed in Plone 7)",
    get_date_options='plone.app.z3cform.widgets.datetime:get_date_options',
    get_datetime_options='plone.app.z3cform.widgets.datetime:get_date_options',
)
zope.deferredimport.deprecated(
    "Import from plone.app.z3cform.widgets.relateditems instead (this package will be removed in Plone 7)",
    get_relateditems_options='plone.app.z3cform.widgets.relateditems:get_relateditems_options',
)
zope.deferredimport.deprecated(
    "Import from plone.app.z3cform.widgets.querystring instead (this package will be removed in Plone 7)",
    get_querystring_options='plone.app.z3cform.widgets.querystring:get_querystring_options',
)
zope.deferredimport.deprecated(
    "Import from plone.app.z3cform.widgets.richtext instead (this package will be removed in Plone 7)",
    get_tinymce_options='plone.app.z3cform.widgets.richtext:get_tinymce_options',
)
zope.deferredimport.deprecated(
    "Import from plone.app.z3cform.utils instead (this package will be removed in Plone 7)",
    get_portal='plone.app.z3cform.utils:get_portal',
    get_portal_url='plone.app.z3cform.utils:get_portal_url',
    get_context_url='plone.app.z3cform.utils:get_context_url',
    get_Widget_form='plone.app.z3cform.utils:get_Widget_form',
)



# -*- coding: utf-8 -*-
import zope.deferredimport

# FileFactory only needed by ATContentTypes
zope.deferredimport.initialize()
zope.deferredimport.deprecated(
    'Import IFileFactory from zope.filerepresentation.interfaces instead',
    IFileFactory='zope.filerepresentation.interfaces:IFileFactory',
)

zope.deferredimport.deprecated(
    'Import IFieldPermissionChecker from plone.app.z3cform.interfaces instead',
    IFieldPermissionChecker='plone.app.z3cform.interfaces:IFieldPermissionChecker',  # noqa
)



# -*- coding: utf-8 -*-
import zope.deferredimport


zope.deferredimport.deprecated(
    "Import from plone.app.z3cform.utils instead (this package will be removed in Plone 7)",
    el_attrib='plone.app.z3cform.utils:el_attrib',
    dict_merge='plone.app.z3cform.utils:dict_merge',
)
zope.deferredimport.deprecated(
    "Import from plone.app.z3cform.widgets.patterns instead (this package will be removed in Plone 7)",
    BaseWidget='plone.app.z3cform.widgets.patterns:BaseWidget',
    InputWidget='plone.app.z3cform.widgets.patterns:InputWidget',
    SelectWidget='plone.app.z3cform.widgets.patterns:SelectWidget',
    TextareaWidget='plone.app.z3cform.widgets.patterns:TextareaWidget',
)



# -*- coding: utf-8 -*-
import zope.deferredimport

# can be removed in Plone 6
zope.deferredimport.initialize()
zope.deferredimport.deprecated(
    'Import Zope2FileUploadStorable from plone.app.z3cform.factories instead',
    Zope2FileUploadStorable='plone.app.z3cform.factories:Zope2FileUploadStorable',
)



# -*- coding: utf-8 -*-

try:
    import unittest2 as unittest
except ImportError:  # pragma: nocover
    import unittest  # pragma: nocover
    assert unittest  # pragma: nocover


class BaseWidgetTests(unittest.TestCase):
    """Tests for plone.app.widgets.base.BaseWidget."""

    def test_defaults(self):
        from plone.app.widgets.base import BaseWidget

        widget = BaseWidget('input', 'example1')
        self.assertEqual(
            widget.render(),
            u'<input class="pat-example1"/>')

        self.assertEqual(widget.klass, 'pat-example1')

    def test_different_element_tag(self):
        from plone.app.widgets.base import BaseWidget

        widget = BaseWidget('select', 'example1')
        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1"/>')

        self.assertEqual(widget.klass, 'pat-example1')

    def test_setting_patterns_options(self):
        from plone.app.widgets.base import BaseWidget

        widget = BaseWidget(
            'input',
            'example1',
            pattern_options={
                'option1': 'value1',
                'option2': 'value2',
            })

        html = widget.render()
        # the order of options is non-deterministic
        result1 = u'<input class="pat-example1" data-pat-example1="{&quot;option1&quot;: &quot;value1&quot;, &quot;option2&quot;: &quot;value2&quot;}"/>'  # noqa: E501
        result2 = u'<input class="pat-example1" data-pat-example1="{&quot;option2&quot;: &quot;value2&quot;, &quot;option1&quot;: &quot;value1&quot;}"/>'  # noqa: E501
        self.assertIn(html, [result1, result2])


class InputWidgetTests(unittest.TestCase):
    """Tests for plone.app.widgets.base.InputWidget."""

    def test_defaults(self):
        from plone.app.widgets.base import InputWidget

        widget = InputWidget('example1', name='example2')

        self.assertEqual(
            widget.render(),
            '<input class="pat-example1" type="text" name="example2"/>')

        self.assertEqual(widget.type, 'text')
        self.assertEqual(widget.value, None)

    def test_set_type_and_value(self):
        from plone.app.widgets.base import InputWidget

        widget = InputWidget(
            'example1',
            name='example2',
            type='email',
            value='example3'
        )

        self.assertEqual(
            widget.render(),
            u'<input class="pat-example1" type="email" '
            'name="example2" value="example3"/>')

        self.assertEqual(widget.type, 'email')
        self.assertEqual(widget.value, 'example3')

        widget.type = 'text'
        widget.value = 'example4'
        self.assertEqual(
            widget.render(),
            u'<input class="pat-example1" type="text" '
            'name="example2" value="example4"/>')

        self.assertEqual(widget.type, 'text')
        self.assertEqual(widget.value, 'example4')

        del widget.type
        del widget.value
        self.assertEqual(
            widget.render(),
            u'<input class="pat-example1" name="example2"/>')

        self.assertEqual(widget.type, None)
        self.assertEqual(widget.value, None)


class SelectWidgetTests(unittest.TestCase):
    """Tests for plone.app.widgets.base.SelectWidget."""

    def test_defaults(self):
        from plone.app.widgets.base import SelectWidget

        widget = SelectWidget('example1', name='example2')

        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1" name="example2"></select>')
        self.assertEqual(list(widget.items), [])
        self.assertEqual(widget.value, [])

    def test_set_items_and_value(self):
        from plone.app.widgets.base import SelectWidget

        items = [
            ('token1', 'value1'),
            ('token2', 'value2'),
            ('token3', 'value3'),
        ]
        widget = SelectWidget(
            'example1',
            name='example2',
            value='token2',
            items=items
        )

        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1" name="example2">'
            '<option value="token1">value1</option>'
            '<option value="token2" selected="selected">value2</option>'
            '<option value="token3">value3</option>'
            '</select>')

        self.assertEqual(list(widget.items), items)
        self.assertEqual(widget.value, ['token2'])

        widget.value = 'token1'
        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1" name="example2">'
            '<option value="token1" selected="selected">value1</option>'
            '<option value="token2">value2</option>'
            '<option value="token3">value3</option>'
            '</select>')

        self.assertEqual(list(widget.items), items)
        self.assertEqual(widget.value, ['token1'])

        del widget.value
        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1" name="example2">'
            '<option value="token1">value1</option>'
            '<option value="token2">value2</option>'
            '<option value="token3">value3</option>'
            '</select>')

        del widget.items
        self.assertEqual(
            widget.render(),
            '<select class="pat-example1" name="example2"></select>')

    def test_multiple(self):
        from plone.app.widgets.base import SelectWidget

        items = [
            ('token1', 'value1'),
            ('token2', 'value2'),
            ('token3', 'value3'),
        ]
        widget = SelectWidget(
            'example1',
            name='example2',
            value=['token2'],
            items=items,
            multiple=True,
        )

        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1" multiple="multiple" name="example2">'
            '<option value="token1">value1</option>'
            '<option value="token2" selected="selected">value2</option>'
            '<option value="token3">value3</option>'
            '</select>')

        self.assertEqual(list(widget.items), items)
        self.assertEqual(widget.value, ['token2'])

        widget.value = ['token1', 'token2']
        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1" multiple="multiple" name="example2">'
            '<option value="token1" selected="selected">value1</option>'
            '<option value="token2" selected="selected">value2</option>'
            '<option value="token3">value3</option>'
            '</select>')

        self.assertEqual(list(widget.items), items)
        self.assertEqual(widget.value, ['token1', 'token2'])

        del widget.value
        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1" multiple="multiple" name="example2">'
            '<option value="token1">value1</option>'
            '<option value="token2">value2</option>'
            '<option value="token3">value3</option>'
            '</select>')

        del widget.items
        self.assertEqual(
            widget.render(),
            u'<select class="pat-example1" multiple="multiple" '
            'name="example2"></select>')


class TextareaWidgetTests(unittest.TestCase):
    """Tests for plone.app.widgets.base.TextareaWidget."""

    def test_defaults(self):
        from plone.app.widgets.base import TextareaWidget

        widget = TextareaWidget('example1', name="example2")
        self.assertEqual(
            widget.render(),
            u'<textarea class="pat-example1" name="example2"></textarea>')

        self.assertEqual(widget.name, 'example2')
        self.assertEqual(widget.klass, 'pat-example1')
        self.assertEqual(widget.value, '')

    def test_setting_patterns_options(self):
        from plone.app.widgets.base import TextareaWidget

        widget = TextareaWidget(
            'example1',
            name='example2',
            pattern_options={
                'option1': 'value1',
                'option2': 'value2',
            })

        html = widget.render()
        # the order of options is non-deterministic
        result1 = u'<textarea class="pat-example1" name="example2" data-pat-example1="{&quot;option1&quot;: &quot;value1&quot;, &quot;option2&quot;: &quot;value2&quot;}"></textarea>'  # noqa: E501
        result2 = u'<textarea class="pat-example1" name="example2" data-pat-example1="{&quot;option2&quot;: &quot;value2&quot;, &quot;option1&quot;: &quot;value1&quot;}"></textarea>'  # noqa: E501
        self.assertIn(html, [result1, result2])

    def test_set_value(self):
        from plone.app.widgets.base import TextareaWidget

        widget = TextareaWidget('example1', name="example2", value='example3')
        self.assertEqual(
            widget.render(),
            u'<textarea class="pat-example1" name="example2">'
            'example3'
            '</textarea>')

        self.assertEqual(widget.value, 'example3')

        widget.value = 'example4'
        self.assertEqual(
            widget.render(),
            u'<textarea class="pat-example1" name="example2">'
            'example4'
            '</textarea>')

        del widget.value
        self.assertEqual(
            widget.render(),
            u'<textarea class="pat-example1" name="example2"></textarea>')

    def test_can_not_change_element_tag(self):
        from plone.app.widgets.base import TextareaWidget
        self.assertRaises(
            TypeError,
            TextareaWidget, 'example1', el='input', name='example2')



# -*- coding: utf-8 -*-
from mock import patch
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.widgets.testing import PLONEAPPWIDGETS_INTEGRATION_TESTING
from plone.app.widgets.utils import get_querystring_options
from plone.app.widgets.utils import get_relateditems_options
from plone.app.widgets.utils import get_tinymce_options

import unittest


class TestQueryStringOptions(unittest.TestCase):
    layer = PLONEAPPWIDGETS_INTEGRATION_TESTING

    def setUp(self):
        setRoles(self.layer['portal'], TEST_USER_ID, ['Contributor'])

    def test__query_string_options(self):
        """Test query string options on root:
        All URLs and paths equal root url and path,
        no favorites
        """

        portal = self.layer['portal']
        options = get_querystring_options(
            portal,
            '@@qsOptions'
        )

        # Test base options
        self.assertEqual(
            options['indexOptionsUrl'],
            'http://nohost/plone/@@qsOptions'
        )

        self.assertEqual(
            options['previewCountURL'],
            'http://nohost/plone/@@querybuildernumberofresults'
        )

        self.assertEqual(
            options['previewURL'],
            'http://nohost/plone/@@querybuilder_html_results'
        )

        # Test options of the AJAX select widget
        self.assertEqual(
            options['patternAjaxSelectOptions']['separator'],
            ';'
        )

        # Test options of the date picker
        self.assertEqual(
            options['patternDateOptions'],
            {'behavior': 'native', 'week-numbers': 'show', 'first-day': 0, 'today': 'Today', 'clear': 'Clear'}
        )

        # Test options of the related items widget
        self.assertEqual(
            options['patternRelateditemsOptions']['basePath'],
            '/plone'
        )
        self.assertTrue(
            'recentlyUsed' not in options['patternRelateditemsOptions']
        )


class TestRelatedItemsOptions(unittest.TestCase):
    layer = PLONEAPPWIDGETS_INTEGRATION_TESTING

    def setUp(self):
        setRoles(self.layer['portal'], TEST_USER_ID, ['Contributor'])

    def test__base_relateditems_options(self):
        """Test related items options on root:
        All URLs and paths equal root url and path,
        no favorites
        """

        portal = self.layer['portal']
        options = get_relateditems_options(
            portal,
            None,
            '#!@',
            'test_vocab',
            '@@vocab',
            'testfield'
        )

        # vocab is correctly set
        self.assertTrue(
            '@@vocab?name=test_vocab&field=testfield'
            in options['vocabularyUrl']
        )

        # rootUrl contains something
        self.assertTrue(
            bool(options['rootUrl'])
        )

        root_path = context_path = '/'.join(portal.getPhysicalPath())
        root_url = context_url = portal.absolute_url()

        # context_path contains something, otherwise this test is meaningless
        self.assertTrue(bool(context_path))
        # context_url contains something, otherwise this test is meaningless
        self.assertTrue(bool(context_url))

        self.assertEqual(
            options['rootUrl'],
            root_url
        )

        self.assertEqual(
            options['rootPath'],
            root_path
        )

        self.assertEqual(
            options['vocabularyUrl'],
            root_url + '/@@vocab?name=test_vocab&field=testfield'
        )

        self.assertEqual(
            options['basePath'],
            context_path
        )

        self.assertEqual(
            options['contextPath'],
            context_path
        )

        self.assertEqual(
            options['separator'],
            '#!@'
        )

        self.assertTrue(
            'favorites' not in options
        )

        # Recently used is configured, but off per default.
        self.assertEqual(
            options['recentlyUsed'],
            False
        )
        self.assertEqual(
            options['recentlyUsedKey'],
            'relateditems_recentlyused_testfield_' + TEST_USER_ID
        )

    def test__subfolder_relateditems_options(self):
        """Test related items options on subfolder:
        Vocab called on root, start path is folder, have favorites.
        """

        portal = self.layer['portal']
        portal.invokeFactory('Folder', 'sub')
        sub = portal.sub
        options = get_relateditems_options(
            sub,
            None,
            '#!@',
            'test_vocab',
            '@@vocab',
            'testfield'
        )

        # vocab is correctly set
        self.assertTrue(
            '@@vocab?name=test_vocab&field=testfield'
            in options['vocabularyUrl']
        )

        # rootUrl contains something
        self.assertTrue(
            bool(options['rootUrl'])
        )

        root_path = '/'.join(portal.getPhysicalPath())
        root_url = portal.absolute_url()
        context_path = '/'.join(sub.getPhysicalPath())
        context_url = sub.absolute_url()

        # context_path contains something, otherwise this test is meaningless
        self.assertTrue(bool(context_path))
        # context_url contains something, otherwise this test is meaningless
        self.assertTrue(bool(context_url))

        self.assertEqual(
            options['rootUrl'],
            root_url
        )

        self.assertEqual(
            options['rootPath'],
            root_path
        )

        self.assertEqual(
            options['vocabularyUrl'],
            root_url + '/@@vocab?name=test_vocab&field=testfield'
        )

        self.assertEqual(
            options['basePath'],
            context_path
        )

        self.assertEqual(
            options['contextPath'],
            context_path
        )

        self.assertEqual(
            options['separator'],
            '#!@'
        )

        self.assertEqual(
            len(options['favorites']),
            2
        )

        self.assertEqual(
            sorted(options['favorites'][0].keys()),
            ['path', 'title']
        )

    def test__subdocument_relateditems_options(self):
        """Test related items options on subdoc:
        Vocab called on root, start path is root as document is not folderish,
        no favorites.
        """

        portal = self.layer['portal']
        portal.invokeFactory('Document', 'sub')
        sub = portal.sub
        options = get_relateditems_options(
            sub,
            None,
            '#!@',
            'test_vocab',
            '@@vocab',
            'testfield'
        )

        # vocab is correctly set
        self.assertTrue(
            '@@vocab?name=test_vocab&field=testfield'
            in options['vocabularyUrl']
        )

        # rootUrl contains something
        self.assertTrue(
            bool(options['rootUrl'])
        )

        root_path = '/'.join(portal.getPhysicalPath())
        root_url = portal.absolute_url()
        context_path = '/'.join(sub.getPhysicalPath())
        context_url = sub.absolute_url()

        # context_path contains something, otherwise this test is meaningless
        self.assertTrue(bool(context_path))
        # context_url contains something, otherwise this test is meaningless
        self.assertTrue(bool(context_url))

        self.assertEqual(
            options['rootUrl'],
            root_url
        )

        self.assertEqual(
            options['rootPath'],
            root_path
        )

        self.assertEqual(
            options['vocabularyUrl'],
            root_url + '/@@vocab?name=test_vocab&field=testfield'
        )

        self.assertEqual(
            options['basePath'],
            root_path
        )

        self.assertEqual(
            options['contextPath'],
            context_path
        )

        self.assertEqual(
            options['separator'],
            '#!@'
        )

        self.assertTrue(
            'favorites' not in options
        )


class TestTinyMCEOptions(unittest.TestCase):
    layer = PLONEAPPWIDGETS_INTEGRATION_TESTING

    def setUp(self):
        setRoles(self.layer['portal'], TEST_USER_ID, ['Contributor'])

    def test__tinymce_options_different_contexts(self):
        """Test if ``get_tinymce_options`` can be called with different
        contexts, including invalid and form contexts.
        """
        request = self.layer['request']
        portal = self.layer['portal']
        portal.invokeFactory('Folder', 'sub')
        sub = portal.sub

        # TinyMCE on portal context
        options = get_tinymce_options(portal, None, request)
        self.assertEqual(options['relatedItems']['basePath'], '/plone')

        # TinyMCE on sub folder context
        options = get_tinymce_options(sub, None, request)
        self.assertEqual(options['relatedItems']['basePath'], '/plone/sub')



from setuptools import setup, find_packages

version = '5.0.0'

setup(
    name='plone.app.widgets',
    version=version,
    description="better plone widgets",
    long_description='%s\n%s' % (
        open("README.rst").read(),
        open("CHANGES.rst").read(),
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    keywords='plone widgets z3cform',
    author='Nathan Van Gheem',
    author_email='vangheem@gmail.com',
    url='https://github.com/plone/plone.app.widgets',
    license='GPL',
    packages=find_packages(),
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    python_requires='>=3.8',
    install_requires=[
        'setuptools',
        # needed because users vocabulary was added here
        'plone.app.vocabularies>=2.1.12',
        'plone.base',
        'six',
    ],
    extras_require={
        'test': [
            'plone.app.robotframework[debug]',
            'plone.app.testing>=4.2.4',  # we need ROBOT_TEST_LEVEL
            'mock',
        ],
        'dexterity': [
            'pytz',
            'plone.app.dexterity',
            'plone.app.contenttypes>=1.1',
            'plone.app.event>=1.2',
        ],
    },
)
